@@ -167,7 +167,7 @@ FancyTabBar::event(QEvent *event) {
 
 // Resets hover animation on mouse enter
 void
-FancyTabBar::enterEvent(QEvent *) {
+FancyTabBar::enterEvent(MtxQEnterEventArgType *) {
   m_hoverRect = QRect();
   m_hoverIndex = -1;
 }
@@ -219,7 +219,7 @@ FancyTabBar::tabRect(int index)
 // mode itself
 void
 FancyTabBar::emitCurrentIndex() {
-  emit currentChanged(m_currentIndex);
+  Q_EMIT currentChanged(m_currentIndex);
 }
 
 void
@@ -326,7 +326,7 @@ void
 FancyTabBar::insertTab(int index,
                        QIcon const &icon,
                        QString const &label) {
-  auto tab    = new FancyTab(this);
+  auto tab    = std::make_shared<FancyTab>(this);
   tab->m_icon = icon;
   tab->m_text = label;
   m_tabs.insert(index, tab);
@@ -334,8 +334,7 @@ FancyTabBar::insertTab(int index,
 
 void
 FancyTabBar::removeTab(int index) {
-  auto tab = m_tabs.takeAt(index);
-  delete tab;
+  m_tabs.removeAt(index);
 }
 
 void
@@ -343,7 +342,7 @@ FancyTabBar::setCurrentIndex(int index) {
   if (isTabEnabled(index)) {
     m_currentIndex = index;
     update();
-    emit currentChanged(m_currentIndex);
+    Q_EMIT currentChanged(m_currentIndex);
   }
 }
 
@@ -430,11 +429,11 @@ FancyTabWidget::FancyTabWidget(QWidget *parent)
   m_selectionWidget = new QWidget(this);
   QVBoxLayout *selectionLayout = new QVBoxLayout;
   selectionLayout->setSpacing(0);
-  selectionLayout->setMargin(0);
+  selectionLayout->setContentsMargins(0, 0, 0, 0);
 
   // StyledBar *bar = new StyledBar;
   // QHBoxLayout *layout = new QHBoxLayout(bar);
-  // layout->setMargin(0);
+  // layout->setContentsMargins(0, 0, 0, 0);
   // layout->setSpacing(0);
   // layout->addWidget(new FancyColorButton(this));
   // selectionLayout->addWidget(bar);
@@ -449,7 +448,7 @@ FancyTabWidget::FancyTabWidget(QWidget *parent)
 
   QVBoxLayout *cornerWidgetLayout = new QVBoxLayout;
   cornerWidgetLayout->setSpacing(0);
-  cornerWidgetLayout->setMargin(0);
+  cornerWidgetLayout->setContentsMargins(0, 0, 0, 0);
   cornerWidgetLayout->addStretch();
   m_cornerWidgetContainer->setLayout(cornerWidgetLayout);
 
@@ -460,13 +459,13 @@ FancyTabWidget::FancyTabWidget(QWidget *parent)
   // m_statusBar->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Fixed);
 
   QVBoxLayout *vlayout = new QVBoxLayout;
-  vlayout->setMargin(0);
+  vlayout->setContentsMargins(0, 0, 0, 0);
   vlayout->setSpacing(0);
   vlayout->addLayout(m_modesStack);
   // vlayout->addWidget(m_statusBar);
 
   QHBoxLayout *mainLayout = new QHBoxLayout;
-  mainLayout->setMargin(0);
+  mainLayout->setContentsMargins(0, 0, 0, 0);
   mainLayout->setSpacing(1);
   mainLayout->addWidget(m_selectionWidget);
   mainLayout->addLayout(vlayout);
@@ -578,9 +577,9 @@ FancyTabWidget::setCurrentIndex(int index) {
 
 void
 FancyTabWidget::showWidget(int index) {
-  emit currentAboutToShow(index);
+  Q_EMIT currentAboutToShow(index);
   m_modesStack->setCurrentIndex(index);
-  emit currentChanged(index);
+  Q_EMIT currentChanged(index);
 }
 
 void
