@@ -7,6 +7,7 @@ char const * const s_grpDerivingTrackLanguagesFromFileNames       = "derivingTra
 char const * const s_grpFileColors                                = "fileColors";
 char const * const s_grpHeaderViewManager                         = "headerViewManager";
 char const * const s_grpInfo                                      = "info";
+char const * const s_grpLanguageShortcuts                         = "languageShortcuts";
 char const * const s_grpRunProgramConfigurations                  = "runProgramConfigurations";
 char const * const s_grpSettings                                  = "settings";
 char const * const s_grpSplitterSizes                             = "splitterSizes";
@@ -58,6 +59,7 @@ char const * const s_valHexPositions                              = "hexPosition
 char const * const s_valHiddenColumns                             = "hiddenColumns";
 char const * const s_valJobRemovalOnExitPolicy                    = "jobRemovalOnExitPolicy";
 char const * const s_valJobRemovalPolicy                          = "jobRemovalPolicy";
+char const * const s_valLanguage                                  = "language";
 char const * const s_valLanguageShortcuts                         = "languageShortcuts";
 char const * const s_valLastConfigDir                             = "lastConfigDir";
 char const * const s_valLastOpenDir                               = "lastOpenDir";
@@ -122,6 +124,7 @@ char const * const s_valSubtitlePolicy                            = "subtitlePol
 char const * const s_valSwitchToJobOutputAfterStarting            = "switchToJobOutputAfterStarting";
 char const * const s_valTabPosition                               = "tabPosition";
 char const * const s_valTrackInfo                                 = "trackInfo";
+char const * const s_valTrackName                                 = "trackName";
 char const * const s_valType                                      = "type";
 char const * const s_valUiDisableDarkStyleSheet                   = "uiDisableDarkStyleSheet";
 char const * const s_valUiDisableHighDPIScaling                   = "uiDisableHighDPIScaling";
